/**********************************************************************************************/
/* FBL.h																					  */
/*                                                                       					  */
/* Foundation Base Library ( FBL )		                                   					  */
/*                                                                       					  */
/* Copyright Paradigma, 1993-2002                                        					  */
/* All Rights Reserved.                                                 					  */
/**********************************************************************************************/

#ifndef __FBL_h
	#define __FBL_h

#if defined(macintosh)
	#include <MacTypes.h>
#endif

#if defined(__MWERKS__)
	#pragma options align=power
#endif

/**********************************************************************************************/
/*										DEFINES:											  */ 
/**********************************************************************************************/
#if defined(__MWERKS__)

	#if macintosh || TARGET_OS_MAC
		#define FBL_MAC 1
	#else
		#define FBL_WIN 1
		#pragma warning(disable : 4068)
	#endif

#endif // __MWERKS__


/**********************************************************************************************/
#if  defined(_MSC_VER) && !defined(__MWERKS__) 

	// Visual Studio C/C++ from Microsoft, Inc.
	#if defined(_M_IX86)
		#define FBL_WIN 		1
	#else
		#error "Not recognized target for Visual C++."
	#endif // _M_IX86

#endif //defined(_MSC_VER) && !defined(__MWERKS__) 


/**********************************************************************************************/
#if defined(__GNUC__) && (defined(__APPLE_CPP__) || defined(__APPLE_CC__) || defined(__NEXT_CPP__))
// gcc based compilers used on OpenStep -> Rhapsody -> Mac OS X
	#define FBL_MAC 		1 
#endif 


/**********************************************************************************************/
/*
   This macro can be used to test the gcc version and can be used like this:

#    if FBL_CHECK_GCC_VERSION(3, 1)
        ... we have gcc 3.1 or later ...
#    else
        ... no gcc at all or gcc < 3.1 ...
#    endif
*/
#define FBL_CHECK_GCC_VERSION( major, minor ) \
    ( defined(__GNUC__) && defined(__GNUC_MINOR__) \
    && ( ( __GNUC__ > (major) ) \
        || ( __GNUC__ == (major) && __GNUC_MINOR__ >= (minor) ) ) )


/**********************************************************************************************/
// For FBL_MAC we need to check if this is INTEL mac. 
//
#if FBL_MAC
	#if __LITTLE_ENDIAN__			// MAC INTEL 
		#define FBL_BIG_ENDIAN		0
		#define FBL_LITTLE_ENDIAN	1
	#else							// MAC PPC 
		#define FBL_BIG_ENDIAN		1
		#define FBL_LITTLE_ENDIAN	0
	#endif //__LITTLE_ENDIAN__
#endif // FBL_MAC	

#if FBL_WIN
		#define FBL_BIG_ENDIAN		0
		#define FBL_LITTLE_ENDIAN	1
#endif // FBL_WIN


/**********************************************************************************************/
// FBL_EXP macro
//
#ifdef FBL_DLL

	#if __GNUC__ 
	
		#if FBL_CHECK_GCC_VERSION(4,0)
			#undef FBL_EXP
			#define FBL_EXP __attribute__((visibility("default")))
		#else	
			#undef FBL_EXP
			#define FBL_EXP
		#endif // FBL_CHECK_GCC_VERSION
	
	#else 
	  
		#define FBL_EXP __declspec(dllexport) 
	 
	#endif // __GNUC__
	
#else // FBL_DLL

	#define FBL_EXP 

#endif // FBL_DLL


/**********************************************************************************************/
/*										TYPES:												  */ 
/**********************************************************************************************/
// Useful short ames for common types
//
typedef unsigned char   		uchar ;
typedef unsigned short  		ushort;
typedef unsigned long   		ulong ;

#if defined(_MSC_VER)
	typedef __int64  			llong;
	typedef unsigned __int64    ullong;
#else
	typedef long long  			llong;
	typedef unsigned long long  ullong;
#endif

//typedef long double 	 		ldouble ;

	// This types specify database parameters: 
#ifndef FBL_LENGTH
#	define FBL_LENGTH 64	
#endif	
	
#if FBL_LENGTH <= 32
	typedef ulong 				flength;	// type for file length < 2GB.
#else 
	typedef ullong 				flength;	// type for file length > 2GB.
#endif

	// Typedef for RecID type:
typedef ulong 					FBL_REC_ID;	// So up to 4 billions records in one Table.	
typedef ulong 					ID_TYPE;	// Some algorithms operate with numbers of THE SAME 
											// type as RecID, but that numbers are not RecIDs.
											// So we define special type for them. 
 

	// For Windows we specify MacOS-common pascal strings types
#ifdef FBL_WIN

	typedef uchar*					StringPtr;
	typedef const uchar*			ConstStringPtr;
	typedef uchar 					Str255[256];
	typedef unsigned char 			Boolean;

	#ifndef NULL
	#ifdef __cplusplus
	#define NULL    0
	#else
	#define NULL    ((void *)0)
	#endif
	#endif

#endif 

	// System indipendent type of File Path:
#if FBL_MAC
	#define FILE_PATH FSSpec
#else 
	#define FILE_PATH char
#endif


/**********************************************************************************************/
enum ELockType
{
	kNoLock = 1,	// Cursor do not set locks and ignore locks of others.
	kReadOnly,		// Cursor set READ ONLY locks on records.
	kReadWrite  	// Cursor set READ WRITE locks. So it change records. 
					// Only for Server side cursors.
};


/**********************************************************************************************/
/*
										DATABASE CONSTANTS:
*/ 
/**********************************************************************************************/

	// Constants for parameter MODE of method DataBase::Create():
enum FBL_DbMode
{
	kFBL_DscDatBlbInd		= 1,	// (description,data,BLOB,indexes)
	kFBL_Dsc_DatBlbInd 		= 2,	// description + (data,BLOB,indexes)
	kFBL_Dsc_DatBlb_Ind		= 3,	// description + (data,BLOB) + indexes
	kFBL_Dsc_Dat_Blb_Ind	= 4,	// description + data + BLOB + indexes
	kFBL_DscDatBlb_Ind		= 5,	// (description,data,BLOB) + indexes
	kFBL_DscDat_Blb_Ind		= 6,	// (description,data) + BLOB + indexes
	kFBL_DscDatInd_Blb		= 7,	// (description,data,indexes) + BLOB 
	kFBL_Dsc_DatInd_Blb		= 8 	// description + (data,indexes) + BLOB 
};
	

	// INTERNAL constants, which are stored in the DB to define native platform.
const uchar kFBL_OsMac				= 1;	
const uchar kFBL_OsWindows			= 2;	
const uchar kFBL_OsUnix				= 4;	

#ifdef FBL_Obsolute
const uchar kFBL_OneFile			= 1;	// (description,data,BLOB,indexes)
const uchar kFBL_TwoFiles 			= 2;	// description + (data,BLOB,indexes)
const uchar kFBL_ThreeFiles			= 3;	// description + (data,BLOB) + indexes
const uchar kFBL_FourFiles 			= 4;	// description + data + BLOB + indexes
#endif


/**********************************************************************************************/
/*
										FIELD CONSTANTS:
*/ 
/**********************************************************************************************/
#define FBL_FIELD_NAME_LEN	32		


	// Types of database fields:
enum	
{
	kFBL_TypeEnum 			= 1,
	kFBL_TypeBoolean		= 2,
	kFBL_TypeByte			= 3,
	kFBL_TypeShort			= 4,
	kFBL_TypeUShort			= 5,
	kFBL_TypeLong			= 6,
	kFBL_TypeULong			= 7,
	kFBL_TypeFloat			= 8,
	kFBL_TypeDouble			= 9,
	kFBL_TypeString			= 10,
	
	kFBL_TypeDate			= 12,	
	kFBL_TypeTime			= 13,		
	kFBL_TypeDateTime		= 14,		

	kFBL_TypeObjectPtr		= 15,	

	kFBL_TypeText			= 17,	// BLOB field which contains text, can be indexed.
	kFBL_TypeSound			= 18,	// reserved
	kFBL_TypePicture		= 19,	
	kFBL_TypeMovie			= 20,	// reserved
	kFBL_TypeBLOB			= 21,

	kFBL_TypeMedium			= 22,
	kFBL_TypeUMedium		= 23,
	
	kFBL_TypeFixedBinary	= 24,
	kFBL_TypeVarBinary		= 25,
	
	kFBL_TypeVarChar		= 26,
	kFBL_TypeLLong			= 27,
	kFBL_TypeULLong			= 28,
	kFBL_TypeLDouble		= 29,
	kFBL_TypeID				= 30	// Virtual field which represent RecID field of VDK_BaseObject.
};


	// The following constants can be used for SetFlags() and GetFlags().
	// You can use combination of this flags for single call.
enum FBL_FieldAttributes	
{
	// r - you can use this flag in CheckFlags() method. Actually you can read any flag.
	// w - you can use this flag in SetFlags()/ClearFlags() methods.

	fNone					= 0,		// (r/w) There is no any flags.
	
	fIndexed      			= 1 << 0,	// (r/w) Field has secondary index.
	fUnique       			= 1 << 1,	// (r/w) Field is unique.

	fRequired   			= 1 << 2,	// (r/ ) Field can't be empty, obsolute.
	fHasRange				= 1 << 3,	// (r/ ) Field has range if user set it (not implemented).
	fNullable				= 1 << 4,   // (r/w) True if the field must support Null values.
	fIndexByWords			= 1 << 5,	// (r/w) For string and text fields only.
	fCompressed				= 1 << 6,	// (r/w) For BLOBs, specify if must be compressed.	

	fReal					= 1 << 8,	// (r/ ) Field is real, i.e. it is saved in the database.
										// Else a field is counted or composed.
								
	fConstant				= 1 << 9,	// (r/ ) Value of field can't be modifyed after initialisation.
								 
	fMethod 				= 1 << 10	// (r/ ) Value of field is counted using values of other fields  
										// of the same record of BaseObject, it is not saved in table,
										// but can have index. You can use this flag only for CheckFlags().
};

	// For methods BaseObject.GetFieldCount() 
enum FBL_FieldKind						
{
	kFBL_All 		= 0,				// Return as Real Fields so Methods of BaseObject.
	kFBL_Fields		= 1,				// Return only Real fields of BaseObject.
	kFBL_Methods	= 2					// Return only Methods of BaseObject.
};

	// Constants for a Picture field (only JPG and WinDIB used now).
enum FBL_PictType
{
	kFBL_Unknown 			= 0,
	kFBL_MacPict 			= 1,
	kFBL_MacCICN			= 2,
	kFBL_MacIconSuite		= 3,
	kFBL_MacGWorld			= 4,
	
	kFBL_WinDIB				= 10,
	
	kFBL_JPG				= 20,
	kFBL_TIFF				= 21,
	kFBL_GIF				= 22
};


	// Header for storing of a disk file in the BLOB field.
	// This header is needed to correctly store ALL info of MacOS files.
#ifdef WINDOWS
	#pragma pack(push, 2)
#endif

struct FBL_OSFileDescription // sizeof = 88 
{
	char		mName[64]; 
	ulong		mCreator;
	ulong		mType;
	ulong		mDataForkLength;
	ulong		mResForkLength;
	ushort		mFlags;
	uchar		mReserved[6];
};


#ifdef WINDOWS
	#pragma pack(pop)
#endif


/**********************************************************************************************/
/*
										RELATED TABLES CONSTANTS:
*/ 
/**********************************************************************************************/
enum FBL_Link_Type
{
	kFBL_ObjectPointer,
	kFBL_BinaryRelation,
	kFBL_RdbRelation
};


typedef ushort	FBL_RelType;		// 0 - Many
									// 1 - One
									// N - not more than N, similiar to Many with the top limit.

	// Constants for FBL_RelType parameters:
const 	ushort 	FBL_Many	= 0;	
const 	ushort	FBL_One		= 1;


	// These constants are inteneded for ObjectPtr field and for RDB-Link (in SQL FOREIGN KEY)
	// They define behavior of deletion of a record from Table A which is related with 
	// Table B as [One : Many].
enum FBL_DeletionControl	
{
	kFBL_SET_NULL			= 0,	// ptr field of all related records of Table B will be set to NULL.
	kFBL_CASCADE			= 1,	// all related records of Table B will be deleted.
	kFBL_RESTRICT			= 2		// can not delete a record 
};


/**********************************************************************************************/
/*
										DEBUG MACROS:
*/
/**********************************************************************************************/
#ifndef __CHECK
	#define __CHECK 0
#endif

void LogAssert( 	
	const char* inFile, 
	const char* inFunc, 
	ulong 		inLine,
	const char* inMsg );

	// Common system-independent and compiler-independent macro CHECK(x) 
#if __CHECK == 0
	#define CHECK(p) // { if(!(p)) LogAssert(__FILE__,__func__,__LINE__,#p); }	
#else
	#ifdef FBL_MAC 						// MacOS:
		#define CHECK(p) Assert_(p)
	#else 								// Windows:
		#ifdef __AFX_H__					// we use MFC
			#define CHECK(p) ASSERT(p)
		#else								// we work without MFC
			#include<crtdbg.h>
			#define CHECK(p) _ASSERTE(p)	
			#define DEBUG_NEW new
		#endif
	#endif
#endif


	// Assert macro used in all sources of kernel
#define __CHECK_FBL 1

#if __CHECK_FBL == 0
	#define CHECK_FBL(p) 
#else
	#define CHECK_FBL(p) CHECK(p)
#endif


	// This macro is usefull for deleting objects of some classes.
	// It shouldn't be used for deletion of array of objects.
#define ForgetObject(obj) \
		(void) ((obj) ? (delete (obj), (obj) = NULL) : 0 )


	// This macros can be used to suppress warning "Parameter is not used".
#define argused1(x1) 			(void)x1;
#define argused2(x1,x2) 		(void)x1;(void)x2;
#define argused3(x1,x2,x3) 		(void)x1;(void)x2;(void)x3;
#define argused4(x1,x2,x3,x4) 	(void)x1;(void)x2;(void)x3;(void)x4;


/**********************************************************************************************/
/*
									GLOBAL FUNCTIONS:
*/
/**********************************************************************************************/

	// Must be called to init Valentina kernel: it creates static members, and then create cache.
	// If you specify serial numbers as NULL, then kernel will work 10 minutes.
	// Returns the actual size of cache it was able to allocate. 
FBL_EXP ulong ValentinaInit( 
				ulong inCacheSize = 3 * 1024 * 1024, 	// in bytes
				char* inMacSN = NULL,					// your MacOS serail number
				char* inWinSN = NULL);					// your Windows serial number.

	// Must be called to finish work with Valentina lernel.
FBL_EXP void ValentinaShutDown( void );


#ifdef FBL_MAC
	inline  void FBL_memcpy( void* dest, const void* src, ulong n){
			 	::BlockMoveData(src, dest, long(n) );
			}
			
	inline void FBL_memmove(void* dest, const void* src, ulong n){ 	
				::BlockMoveData(src, dest, long(n) );
			}
#else 
	inline void FBL_memcpy(void* dest, const void* src, ulong n){ 	
				::memcpy (dest, src, n );
			}
			
	inline void FBL_memmove(void* dest, const void* src, ulong n){ 	
				::memmove(dest, src, n );
			}
#endif


	// This 2 functions will become absolute, so please replace in your code
	// call to this functions on ValentinaInit()/ValentinaShutDown(). 
FBL_EXP ulong MakeDataBaseCache( ulong inBytes = 3 * 1024 * 1024 );
FBL_EXP void DestroyDataBaseCache( void );


/**********************************************************************************************/
template<class T> 
void FBL_Swap( T& a, T& b )
{
	T tmp = a;
	a = b;
	b = tmp;
}

template <class T>
inline const T& FBL_Min(const T& a, const T& b)	{ return a < b ? a : b; }

template <class T>
inline const T& FBL_Max(const T& a, const T& b)	{ return a > b ? a : b; }


template<class T> 
void FBL_Fill( T* p1, T* p2, T c )
{
	while( p1 < p2 )
		*p1++ = c;
}


/**********************************************************************************************/
/*
						TYPES USED IN CURSOR TO BUILD INFORMATION BUFFERS.
*/
/**********************************************************************************************/
struct FBL_CursorFieldDesc			
{
	char	mName[32];
	uchar	mType;

		// Next 2 parameters contains data exactly as in BaseObkect.MakeNewField();
	ulong 	mParam1;	// plus for numeric data, it contains GetBytes() value. 
	ulong	mParam2;	
};

class FBL_CursorDesc		// used by GetCursorDescription()
{
	public://///////////////////////////////////////////////////////////////////////////////////
	 
			 					FBL_CursorDesc( ulong inFldCount, ulong inRecCount ){
			 						mBuffer = new char [ 2 * sizeof(ulong) + 
			 											 inFldCount * sizeof(FBL_CursorFieldDesc) ];
			 						
			 						FieldCount() 	= inFldCount;
			 						RecordCount()	= inRecCount;	
			 						mDesc			= (FBL_CursorFieldDesc*)(mBuffer + 2 * sizeof(ulong));		 
			 					}	
			 					
			 					~FBL_CursorDesc( void )	{ delete [] mBuffer;			}
			 						 					
	 	ulong&					FieldCount( void ) 		{ return ((ulong*)mBuffer)[0]; 	} 				
	 	ulong&					RecordCount( void ) 	{ return ((ulong*)mBuffer)[1]; 	} 				
	 	FBL_CursorFieldDesc&	operator[]( ushort i )	{ return mDesc[i]; 				}			
	 					
	protected:////////////////////////////////////////////////////////////////////////////////// 

		char*					mBuffer;
		FBL_CursorFieldDesc*	mDesc;
};

 
/**********************************************************************************************/
/*
									INTERNAL CONSTANTS:
*/
/**********************************************************************************************/
// Note,  temporary files which must be automatically deleted on open of data file
// must have negative numbers. These are internal numbers of Valentina kernel. 
// C++ Developer can use them only for encryption of database to define the kind of context 
// of the logical file.

enum FBL_EmbededType	// can have range MIN_SHORT .. MAX_SHORT
{
	// EACH data-file have one file's table.
	kFBL_FileTable			= 1,	// internal file, describe file structure. 

	// DESCRIPTION data-file can have:
	kFBL_Domain				= 2,	// File with description of domain.
	kFBL_CustomDomain		= 9,	// For additional description created by C++ developer. 
	kFBL_Descriptors		= 11,	// Valentina DBMS use it: layouts, predefined seach, sort 
	kFBL_SystemTable		= 13,	// System table, stored in the ".vdb" file.
	kFBL_SystemFile2		= 14,	// System second file for BLOB or VarChar field, stored in the ".vdb" file.	

	// DATA data-file:
	kFBL_Table				= 3,	// Table.
	kFBL_Link				= 4,
	kFBL_Nulls				= 10,	// Bit file where we keep NULL values.
	kFBL_VarChar			= 12,   // Second File for VarChar
	
	// BLOB data-file:
	kFBL_BLOB  				= 7,	// Second file for BLOB field.

	// TMP data-file: 
	kFBL_Index				= 5,	// Index for that table
	kFBL_SM					= 6,	// Secondary index.

	kFBL_TmpFile			= -1,	// Temporary mFile, used on type conversion.
	kFBL_LockFile			= -2,	// Bit file with information about record locks.
	kFBL_JoinTable			= -5,	// Temporary file of JoinTable
	kFBL_TmpFile2			= -4,	// Temporary mFile2 of field, used on type conversion to BLOB or VarChar.
									// (it needs if all db is one file, to differ from tmp files of table)

	kFBL_TmpTable			= -kFBL_Table,		// - 3,	Fields of tmp table.
	kFBL_TmpBLOB  			= -kFBL_BLOB,		// - 7,	BLOB fields of tmp table.
	kFBL_TmpNulls			= -kFBL_Nulls,		// -10, Nulls file of tmp table.
	kFBL_TmpVarChar			= -kFBL_VarChar		// -12, VarChar fields of tmp table. 

};


/**********************************************************************************************/
// The FBL reserve STR# resources 2001..2049 (MacOS)
const ulong kFBL_ErrorsSTRid	= 2002;		
const ulong kFBL_TypesSTRid 	= 2001;		
	

/**********************************************************************************************/
/*
										TABLE CONSTANTS:
*/ 
/**********************************************************************************************/
// Read/Write flags of record locks.
//
const ulong kLockFlagRead  = ulong(1 << 31);
const ulong kLockFlagWrite = ulong(1 << 30); 

// Extract form ulong value the owner ID removing 2 flag bits.
inline ulong ExtractOwner( ulong inValue ) 
{ 
	return (inValue << 2) >> 2; 
}


/**********************************************************************************************/
//	std Namespace

#ifndef FBL_Uses_Std_Namespace
	#define	FBL_Uses_Std_Namespace			1		// Default to ON
#endif

#if FBL_Uses_Std_Namespace
	// empty declaration of std to avoid error in case MSL has not space name.
	namespace std { void ___dummyf(void); }

	#define FBL_Using_Namespace_Std			using namespace std;
	#define FBL_STD							std
#else
	#define FBL_Using_Namespace_Std
	#define FBL_STD
#endif


/**********************************************************************************************/
// Some common includes:
#if VSDK_FRAMEWORK_STYLE
#	include <VSDK/FBL_Array.h>
#	include <VSDK/FBL_Exceptions.h>
#	include <VSDK/FBL_Deleter.h>
#	include <VSDK/FBL_Sharable.h>
#else
#	include <FBL_Array.h>
#	include <FBL_Exceptions.h>
#	include <FBL_Deleter.h>
#	include <FBL_Sharable.h>
#endif // VSDK_FRAMEWORK_STYLE


/**********************************************************************************************/
#define FBL_CLASS_DEF(classname)\
	typedef FBL_TSharablePtr<classname>	classname##CPtr			


/**********************************************************************************************/
#if defined(__MWERKS__)
	#pragma options align=reset
#endif

#endif	// __FBL_h