/**********************************************************************************************/
/* FBL_Exceptions.h																		 	  */
/*                                                                       					  */
/* Foundation Base Library ( FBL )		                                   					  */
/*                                                                       					  */
/* Copyright Paradigma, 1993-2002                                        					  */
/* All Rights Reserved.                                                 					  */
/**********************************************************************************************/

#ifndef __FBL_Exceptions_h 
	#define __FBL_Exceptions_h

#if VSDK_FRAMEWORK_STYLE
#	include <VSDK/FBL.h>
#else
#	include <FBL.h>
#endif // VSDK_FRAMEWORK_STYLE

#include <string.h>

#if defined(__MWERKS__)
	#pragma options align=power
#endif


/**********************************************************************************************/
class FBL_String;


/**********************************************************************************************/
enum FBL_Errors
{
	kFBL_NoError				= 0,
	kFBL_TimeOut				= 666,		

	kFBL_Error					= 300,		

	kFBL_DataFileError			= 301,
	kFBL_TableError				= 302,
	kFBL_FieldError				= 303,
	kFBL_IndexError				= 304,
	kFBL_ParserError			= 305,
	kFBL_CacheIsNotPresent		= 306,		
	kFBL_AssertionFail			= 307,
	kFBL_AssertionNULL			= 308,
	kFBL_CacheIsSmall			= 309,
	kFBL_WrongParameterType		= 310,
	kFBL_MemFull				= 311,		
	
											// DataFile errors:
	kFBL_CantOpenNewVersion		= 320,
	kFBL_CantReadDomain			= 321,
	kFBL_EmbFileNotFound		= 322,
	kFBL_BaseObjectNotFound		= 323,
	kFBL_FieldNotFound			= 324,
	kFBL_WrongPassword			= 325,
	kFBL_WrongEncryptionKey		= 326,
	kFBL_DatabaseIsNotOpened	= 327,
	kFBL_WrongStructureEncryptionKey = 328,
	
											// Field errors.
	kFBL_FieldNotIndexed		= 340,		
	kFBL_FieldIsConstant		= 341,
	kFBL_FieldIsCounted			= 342,
	kFBL_FieldIsComposed		= 343,
	kFBL_FieldIsUnique			= 344,
	kFBL_IndexNotSorted			= 345,
	kFBL_FieldNameNotUnique		= 346,
	kFBL_FieldIsNotMethod		= 347,		
	kFBL_InvPageVarChar			= 348,
	kFBL_CannotCreate			= 349,
	kFBL_LanguageNotFound		= 350,
	kFBL_CorruptedVarChar		= 351,	

											// Table errors.
	kFBL_TableIsEmpty			= 360,		
	kFBL_TableIsCorrupted		= 361,
	kFBL_RecordNotFound			= 362,
	kFBL_CannotSetLock			= 363,
	
											// Index errors.
	kFBL_SXCorrupted			= 370,
	kFBL_FailToBuild			= 371,
	kFBL_InvPage				= 372,
	 		
											// query parser errors.
	kFBL_NeedValue				= 380,		
	kFBL_NeedLeftBound			= 381,
	kFBL_NeedRightBound			= 382,
	
											// Parameter errors.
	kFBL_WrongDataBaseRef		= 390,
	kFBL_WrongBaseObjectRef		= 391,
	kFBL_WrongFieldRef			= 392,
	kFBL_WrongCursorRef			= 393,
	kFBL_WrongDbInfoRef			= 394,
	kFBL_WrongClientInfoRef		= 395

};


/**********************************************************************************************/
/*
	 This is a normal exception class to be used in the kernel.
	 You can always catch xFBL_Exception exception to catch ANY exception from database kernel. 
*/
class FBL_EXP xFBL_Exception 			
{
	public://///////////////////////////////////////////////////////////////////////////////////
	
FBL_EXP						xFBL_Exception( long inErrorCode = kFBL_Error );												
FBL_EXP						xFBL_Exception( long inErrorCode, const char* inErrStr, ulong inStrLen = 0 );
FBL_EXP						xFBL_Exception( const xFBL_Exception& inExc );
							
FBL_EXP virtual				~xFBL_Exception( void );
												
		long				GetErrorCode( void ) const 			{ return mErrorCode; }
		void				SetErrorCode( long inErrorCode )	{ mErrorCode = inErrorCode; }

#ifdef FBL_MAC 	
FBL_EXP	virtual void 		LoadErrorString( Str255 ioStr ) const;				
#endif 	
FBL_EXP virtual void 		LoadErrorString( FBL_String* ioStr ) const;				

						
	protected://///////////////////////////////////////////////////////////////////////////
	
		long				mErrorCode;
		char*				mErrorStr;
		bool				mOwnStr;							
};


/**********************************************************************************************/
#define DECLARE_EXCEPTION(NewExc, BaseExc, kErr)				\
class NewExc : public BaseExc 						\
{																\
	 															\
	public:														\
		NewExc( long err = kErr ) : BaseExc( err ) {}			\
};


/**********************************************************************************************/
#define IMPLEMENT_EXCEPTION(NewExc, BaseExc, kErr)				\
NewExc::NewExc( long err ) : BaseExc( err ) {}			


/**********************************************************************************************/
DECLARE_EXCEPTION( xFBL_TimeOut				, xFBL_Exception	, kFBL_TimeOut			)
DECLARE_EXCEPTION( xFBL_DataFileError		, xFBL_Exception 	, kFBL_DataFileError	)
DECLARE_EXCEPTION( xFBL_TableError			, xFBL_Exception 	, kFBL_TableError		)
DECLARE_EXCEPTION( xFBL_FieldError			, xFBL_Exception 	, kFBL_FieldError		)	
DECLARE_EXCEPTION( xFBL_IndexError			, xFBL_Exception 	, kFBL_IndexError		)
DECLARE_EXCEPTION( xFBL_ParserError			, xFBL_Exception 	, kFBL_ParserError		)
DECLARE_EXCEPTION( xFBL_CacheIsSmall		, xFBL_Exception 	, kFBL_CacheIsSmall		)
DECLARE_EXCEPTION( xFBL_WrongParameterType	, xFBL_Exception 	, kFBL_WrongParameterType)


/**********************************************************************************************/
DECLARE_EXCEPTION( xFBL_CantOpenNewVersion	, xFBL_DataFileError, kFBL_CantOpenNewVersion)
DECLARE_EXCEPTION( xFBL_CantReadDomain		, xFBL_DataFileError, kFBL_CantReadDomain	)
DECLARE_EXCEPTION( xFBL_EmbFileNotFound		, xFBL_DataFileError, kFBL_EmbFileNotFound	)
DECLARE_EXCEPTION( xFBL_WrongEncryptionKey	, xFBL_DataFileError, kFBL_WrongEncryptionKey	)
DECLARE_EXCEPTION( xFBL_DatabaseIsNotOpened	, xFBL_DataFileError, kFBL_DatabaseIsNotOpened	)
DECLARE_EXCEPTION( xFBL_WrongStructureEncryptionKey, xFBL_DataFileError, kFBL_WrongStructureEncryptionKey	)


/**********************************************************************************************/
DECLARE_EXCEPTION( xFBL_FieldNotIndexed		, xFBL_FieldError 	, kFBL_FieldNotIndexed	)
DECLARE_EXCEPTION( xFBL_FieldIsConstant		, xFBL_FieldError 	, kFBL_FieldIsConstant	)
DECLARE_EXCEPTION( xFBL_FieldIsCounted		, xFBL_FieldError 	, kFBL_FieldIsCounted	)
DECLARE_EXCEPTION( xFBL_FieldIsComposed		, xFBL_FieldError 	, kFBL_FieldIsComposed	)
DECLARE_EXCEPTION( xFBL_FieldIsUnique		, xFBL_FieldError 	, kFBL_FieldIsUnique	)
DECLARE_EXCEPTION( xFBL_IndexNotSorted		, xFBL_FieldError 	, kFBL_IndexNotSorted	)
DECLARE_EXCEPTION( xFBL_FieldIsNotMethod	, xFBL_FieldError 	, kFBL_FieldIsNotMethod	)
DECLARE_EXCEPTION( xFBL_InvPageVarChar		, xFBL_FieldError 	, kFBL_InvPageVarChar	)
DECLARE_EXCEPTION( xFBL_CannotCreate		, xFBL_FieldError 	, kFBL_CannotCreate	)
DECLARE_EXCEPTION( xFBL_LanguageNotFound	, xFBL_FieldError 	, kFBL_LanguageNotFound	)
DECLARE_EXCEPTION( xFBL_CorruptedVarChar	, xFBL_FieldError 	, kFBL_CorruptedVarChar	)


/**********************************************************************************************/
DECLARE_EXCEPTION( xFBL_TableIsEmpty		, xFBL_TableError 	, kFBL_TableIsEmpty		)
DECLARE_EXCEPTION( xFBL_TableIsCorrupted	, xFBL_TableError 	, kFBL_TableIsCorrupted	)
DECLARE_EXCEPTION( xFBL_RecordNotFound		, xFBL_TableError 	, kFBL_RecordNotFound	)
DECLARE_EXCEPTION( xFBL_CannotSetLock		, xFBL_TableError 	, kFBL_CannotSetLock	)


/**********************************************************************************************/
DECLARE_EXCEPTION( xFBL_SXCorrupted			, xFBL_IndexError  	, kFBL_SXCorrupted		)
DECLARE_EXCEPTION( xFBL_FailToBuild			, xFBL_IndexError  	, kFBL_FailToBuild		)
DECLARE_EXCEPTION( xFBL_InvPage				, xFBL_SXCorrupted 	, kFBL_InvPage			)


/**********************************************************************************************/
DECLARE_EXCEPTION( xFBL_NeedValue			, xFBL_ParserError 	, kFBL_NeedValue		)
DECLARE_EXCEPTION( xFBL_NeedLeftBound		, xFBL_ParserError 	, kFBL_NeedLeftBound	)
DECLARE_EXCEPTION( xFBL_NeedRightBound		, xFBL_ParserError 	, kFBL_NeedRightBound	)


/**********************************************************************************************/
#ifdef FBL_MAC

	#if PP_Debug == 1
		#define FBL_Throw(x) { FBL_DebugThrow(StringLiteral_(__FILE__), __LINE__); throw x; }
	#else
		#define FBL_Throw(x) throw x
	#endif
	
	#define FBL_ThrowIfOSErr(err)												\
		do{																		\
			if (err) { FBL_Throw( xFBL_Exception(err) ); }						\
		}while (false)
	
#else // FBL_MAC

	#define FBL_Throw(x)	throw x

#endif // FBL_MAC

#define	FBL_ThrowIf_(test)														\
	do{																			\
		if (test) { FBL_Throw( xFBL_Exception(kFBL_AssertionFail) ); }			\
	}while (false)

#define	FBL_ThrowIfNil_(ptr)													\
	do{																			\
		if ((ptr) == NULL) { FBL_Throw(xFBL_Exception(kFBL_AssertionNULL) ); }	\
	}while (false)


/**********************************************************************************************/
#if PP_Debug 

	inline void FBL_DebugThrow( ConstStringPtr	inFile,
								long			inLine )
	{
		uchar errStr[] = "\pFBL exception";
	
		if (UDebugging::gDebugThrow == debugAction_Alert) {
#if __PowerPlant__ >= 0x02118000
			UDebugging::AlertThrowAt(errStr, __func__, inFile, inLine);
#else
			UDebugging::AlertThrowAt(errStr, inFile, inLine);
#endif			
		} else if (UDebugging::gDebugThrow == debugAction_Debugger) {
			DebugStr(errStr);
		}
	}
	
#endif // PP_Debug


/**********************************************************************************************/
#if defined(__MWERKS__)
	#pragma options align=reset
#endif

#endif	// __FBL_Exceptions_h
