/**********************************************************************************************/
/* FBL_Formula.h																			  */
/*                                                                       					  */
/* Foundation Base Library ( FBL )		                                   					  */
/*                                                                       					  */
/* Copyright Paradigma, 1993-2002                                        					  */
/* All Rights Reserved.                                                 					  */
/**********************************************************************************************/

#ifndef __FBL_Formula_h
   #define __FBL_Formula_h

#if VSDK_FRAMEWORK_STYLE
#	include <VSDK/FBL.h>
#else
#	include <FBL.h>
#endif // VSDK_FRAMEWORK_STYLE

#include <string.h>

#if defined(__MWERKS__)
	#pragma options align=power
#endif


/**********************************************************************************************/
class FBL_Task;
class FBL_BitSet;


/**********************************************************************************************/
class FBL_Formula 
{
	public://///////////////////////////////////////////////////////////////////////////////////

						FBL_Formula( void ) 	{ mTask = NULL; }
virtual					~FBL_Formula( void ) 	{ }					
							
virtual bool			DoFormula( 
							FBL_REC_ID	inRecID,			// record to proceed
							void*		ioFieldValue,		// pointer to current field's value
							bool&		ioIsNull ) = 0;		// is the field NULL (for nullable fields)
						// Must be overriden in derived class.
						// This function can modify passed value of the field and its flag IsNull. 		
						// You can be intersting in the flag IsNull only if you work with Nullable
						// field and your formula modify records with Null value.

	public://///////////////////////////////////////////////////////////////////////////////////

		FBL_Task*		mTask;		// You can need it if you want cancel a task.
};


/**********************************************************************************************/
class FBL_ConstFormula 
{
	public://///////////////////////////////////////////////////////////////////////////////////

						FBL_ConstFormula( void ) { mTask = NULL; }

virtual					~FBL_ConstFormula( void ) {}			

							
virtual bool			DoFormula( 
							FBL_REC_ID	inRecID,			// record to proceed
							const void* inFieldValue,		// pointer to current field's value
							bool		inIsNull ) = 0;		// is the field NULL (for nullable fields)					
						
	public://///////////////////////////////////////////////////////////////////////////////////

		FBL_Task*		mTask;		// You can need it if you want cancel a task.
};


/**********************************************************************************************/
class FBL_SearchByFormula : public FBL_ConstFormula
{
	public://///////////////////////////////////////////////////////////////////////////////////

		FBL_BitSet*		mResultedSet;	// Here we will accummulate records which match to search.
										// When task finish you must obtain this set to get results. 	

	public://///////////////////////////////////////////////////////////////////////////////////

						// You should create FBL_BitSet according to the number of records
						// in the Table of searched field.
						FBL_SearchByFormula( FBL_BitSet* inResultedSet ) { 
							mResultedSet = inResultedSet;
						}
						
virtual					~FBL_SearchByFormula( void ) {}			
						 	
virtual bool			DoFormula( 
							FBL_REC_ID	inRecID,			// record to proceed
							const void* inFieldValue,		// pointer to current field's value
							bool		inIsNull ) = 0;		// is the field NULL (for nullable fields)					
						
};


/**********************************************************************************************/
#if defined(__MWERKS__)
	#pragma options align=reset
#endif

#endif
