/**********************************************************************************************/
/* FBL_SyncObjects.h																		  */
/*                                                                       					  */
/* Foundation Base Library ( FBL )		                                   					  */
/*                                                                       					  */
/* Copyright Paradigma, 1993-2002															  */
/* All Rights Reserved.                                                 					  */
/**********************************************************************************************/
 
#ifndef __FBL_SyncObjects_h
	#define __FBL_SyncObjects_h

#if VSDK_FRAMEWORK_STYLE
#	include <VSDK/FBL.h>
#	include <VSDK/FBL_SyncObjectsNative.h>
#else
#	include <FBL.h>
#	include <FBL_SyncObjectsNative.h>
#endif // VSDK_FRAMEWORK_STYLE

#include <limits.h>

#if defined(__MWERKS__)
	#pragma options align=power
#endif


/**********************************************************************************************/
class FBL_SyncObject 
{
	public://///////////////////////////////////////////////////////////////////////////////////
	 
								FBL_SyncObject( void )		{}
virtual 						~FBL_SyncObject( void )		{}
						
								/**< Returns true if the current thread was able to acquire the lock. */
virtual bool 					Lock( long inTimeout = LONG_MAX ) 	= 0;
virtual	void 					Unlock( void ) 						= 0;

};


/**********************************************************************************************/
class FBL_Mutex : public FBL_SyncObject 
{
	public://///////////////////////////////////////////////////////////////////////////////////

  								/**< You can specify that the mutex start out locked to avoid
  								race conditions (eg if it's possible for another thread
  								to get a pointer to the mutex before everything has been initialized). */
  								FBL_Mutex( bool inLocked = false ){
  									mMutexNative = FBL_CreateMutex( inLocked );
  								}

virtual 						~FBL_Mutex( void ) {
									ForgetObject( mMutexNative ); 
								}	

								/**< Note that mutexes are recursive: the same thread can re-lock 
								a mutex without blocking. */
virtual bool 					Lock( long inTimeout = LONG_MAX ) { 
									return mMutexNative->Lock(inTimeout); 
								}
								
virtual void 					Unlock( void ) { 
									mMutexNative->Unlock(); 
								}


	protected:////////////////////////////////////////////////////////////////////////////////// 
	
		FBL_MutexNative*		mMutexNative;
													
};


/**********************************************************************************************/
class FBL_Semaphore : public FBL_SyncObject 
{
	public://///////////////////////////////////////////////////////////////////////////////////

		  						/**< The semaphore can be locked count times before blocking.
		  						A semaphore with a count of one acts like a mutex except
		  						that it can be unlocked from any thread (mutex's can only
		  						be unlocked by the thread that locked the mutex). */
		  						FBL_Semaphore( ulong inCount ){
		  							mSemaphoreNative = FBL_CreateSemaphore( inCount );
		  						}

  						
virtual		 					~FBL_Semaphore( void ){
									ForgetObject( mSemaphoreNative );
								}
	
virtual bool 					Lock( long inTimeout = LONG_MAX ){ 
									return mSemaphoreNative->Lock(inTimeout); 
								}
								
virtual void 					Unlock( void ){ 
									mSemaphoreNative->Unlock(); 
								}
									

	protected:////////////////////////////////////////////////////////////////////////////////// 
	
		FBL_SemaphoreNative*	mSemaphoreNative;
};


#pragma mark -


/**********************************************************************************************/
class FBL_StSingleLocker 
{
	public://///////////////////////////////////////////////////////////////////////////////////
	 
								FBL_StSingleLocker( FBL_SyncObject& inObject, long inTimeout = LONG_MAX ) : 
									mObject( inObject )
								{
									mObject.Lock( inTimeout );
								}
									
								~FBL_StSingleLocker( void ){
									mObject.Unlock();
								}

								/**< Returns true if the current thread was able to acquire the lock.
								You should call this if you specified a timeout. */
		bool 					AcquiredLock( void ) const { return mLocked; }
						

	private:////////////////////////////////////////////////////////////////////////////////////
	 
		FBL_SyncObject&			mObject;
		bool					mLocked;
};


/**********************************************************************************************/
#if defined(__MWERKS__)
	#pragma options align=reset
#endif

#endif // __FBL_SyncObjects_h