/**********************************************************************************************/
/* FBL_UseFormula.h	                                                     					  */
/*                                                                       					  */
/* Foundation Base Library ( FBL )															  */
/*																							  */
/* Copyright Paradigma, 1993-2002															  */
/* All Rights Reserved.                                                 					  */
/**********************************************************************************************/

#ifndef __FBL_UseFormula_h 
	#define __FBL_UseFormula_h

#include "FBL_Task.h"
#include "FBL_Progresser.h"
#include "FBL_Thread.h"

#if defined(__MWERKS__)
	#pragma options align=power
#endif


/**********************************************************************************************/
class FBL_BaseField;
class BF_BLOB;
class FBL_Set;
class FBL_ConstFormula;
class FBL_SecondaryIndex;
class FBL_SetIterator;


/**********************************************************************************************/
class FBL_UseFormula : public FBL_Task
{
    public://///////////////////////////////////////////////////////////////////////////////////

		/* initialization */

							FBL_UseFormula( 
								FBL_BaseField*		inField,
								FBL_ConstFormula*	inFormula,
								FBL_Set*	 		inSelection,
								bool				inOwnSelection 		= true,
								const char*			inTaskName 			= NULL,
								bool				inUseBLOBContext 	= true ); 
						
virtual						~FBL_UseFormula( void );
					
							// Main procedure of the thread.
virtual void 				Run( void );
			

    protected://////////////////////////////////////////////////////////////////////////////////
				
		void 				UseFormula( void );
		
		void 				UseFormulaForBlobContext( BF_BLOB* inBLOB );
		void 				UseFormulaForBlobRefs( void );
		void 				UseFormulaForBlobMethod( BF_BLOB* inBLOB );
		void 				UseFormulaForTextMethod( BF_BLOB* inBLOB );
		void 				UseFormulaForField( void );
		void 				UseFormulaForMethod( void );
		
		void				OneDone( void )
							{	
								++mDoneRecords;

								if( mDoneRecords > mNextPercent ){
									mNextPercent += mOnePercent;
									 ++mDonePercent;
									if( mProgresser ) 
										mProgresser->SetValue( mDonePercent );
								}

								FBL_TickYield();			
							}
		

    protected://////////////////////////////////////////////////////////////////////////////////
				
		FBL_BaseField*		mField;				// Field to be modifyed
		FBL_ConstFormula*	mFormula;			// Formula to be applied to the field.
												// Will be destroyed by destructor of task in any case.
		
		FBL_Set* 			mSelection;			// Selection of records to modify.
		bool				mOwnsSelection;	
		
		bool				mUseBLOBContext;	// if TRUE then formula is applied to context of BLOB
												// else to ULONG field in table, i.e. to reference of BLOB.
												// This is useful to find empty BLOBs for example.					

    protected://////////////////////////////////////////////////////////////////////////////////
		
		double 				mOnePercent;
		double 				mNextPercent;
		ulong 				mDoneRecords;
		
		void* 				mpFieldData;
		FBL_SetIterator*	mpIter;
};


/**********************************************************************************************/
#if defined(__MWERKS__)
	#pragma options align=reset
#endif

#endif	// __FBL_UseFormula_h


