/**********************************************************************************************/
/* OBL_Dump.h        		                                                  				  */
/*                                                                       					  */
/* Object Base Library ( OBL )			                                   					  */
/*                                                                       					  */
/* Copyright Paradigma, 1993-2002															  */
/* All Rights Reserved.                                                 					  */
/**********************************************************************************************/
/*
	OBL_Dump -- the abstract class that specify API for dumping.
*/

#ifndef __OBL_Dump_h
	#define __OBL_Dump_h

#if VSDK_FRAMEWORK_STYLE
#	include <VSDK/FBL.h>
#	include <VSDK/FBL_String.h>
#else
#	include <FBL.h>
#	include <FBL_String.h>
#endif // VSDK_FRAMEWORK_STYLE


/**********************************************************************************************/
class FBL_BaseField;
class FBL_File;

class OBL_BaseObject;
class OBL_Cursor;
class OBL_DataBase;


/**********************************************************************************************/
class OBL_Dump
{
	public://///////////////////////////////////////////////////////////////////////////////////
 
		enum Type
		{
			kSQL	= 1,
			kXML	= 2
		};

	public://///////////////////////////////////////////////////////////////////////////////////
 
 			// Creation methods:
 
							OBL_Dump( const FILE_PATH* inFileSpec );
virtual						~OBL_Dump( void ); 
						
			// Dump interface methods:			
						
							// Dump the whole database. its structure and records.
virtual	void				Dump( 
								OBL_DataBase* 	inDataBase, 
								bool 			inDumpHeader = true ) = 0;

							// Dump single BaseObject, its structure and records
virtual	void				Dump( 
								OBL_BaseObject* inBaseObject, 
								bool 			inDumpHeader = true ) = 0;


		FBL_String&			GetResult( void ) { return mStr; }


	protected:////////////////////////////////////////////////////////////////////////////////// 
	
		FBL_File*			mFile;	// file into which we must dump.
		 
		FBL_String			mStr;	// string buffer that we can use for building in RAM
									// some piece of string then write to that file. 
};


/**********************************************************************************************/
// Factory of OBL_Dump class implementation.

FBL_EXP OBL_Dump* OBL_CreateDump( 
	const FILE_PATH* 	inFileSpec,	
	OBL_Dump::Type 		inType );
	
	
FBL_EXP OBL_Dump* OBL_CreateDumpSQL( 
	const FILE_PATH* inFileSpec);

FBL_EXP OBL_Dump* OBL_CreateDumpXML(
	const FILE_PATH* inFileSpec );


/**********************************************************************************************/
#endif // __OBL_Dump_h