/**********************************************************************************************/
/* VDK_Parameter.h																			  */
/*                                                                       					  */
/* Copyright Paradigma, 1993-2003                                        					  */
/* All Rights Reserved.                                                 					  */
/**********************************************************************************************/

#ifndef __VDK_Parameter_h
	#define __VDK_Parameter_h

#if VSDK_FRAMEWORK_STYLE
#	include <VSDK/FBL_Array.h>
#else
#	include <FBL_Array.h>
#endif // VSDK_FRAMEWORK_STYLE


/**********************************************************************************************/
// Abstract class for interfice of bind parameter that we can use in SqlExecute().
// You can create subclass for your own type of data.
// 
class VDK_IParameter 
{
	public://///////////////////////////////////////////////////////////////////////////////////
	 
virtual						~VDK_IParameter( void ) {}	 	

virtual	ulong				GetSize( void ) const = 0;	
virtual const uchar*		GetDataPtr( void ) const = 0;

							// This can return something special.
							// e.g. picture can store here quality.
virtual	long				GetParam( void ) const = 0;

};


/**********************************************************************************************/
typedef FBL_Array<VDK_IParameter*> BindParameters;


/**********************************************************************************************/
// VDK_IParameter implementation;
// Just store the pointer to buffer.
// 
class VDK_Parameter : public VDK_IParameter
{
	public://///////////////////////////////////////////////////////////////////////////////////
	
							VDK_Parameter( void )
							{	
								mSize  = 0;								
								mValue = NULL;
								mCopy  = false;
								mOwnData = false;
							}							
	
							VDK_Parameter( 
								uchar*	inValue, 
								ulong	inSize,
								bool	inDoCopy,
								bool	inOwnData ) 
							{
								SetData( inValue, inSize, inDoCopy, inOwnData );
							}

virtual						~VDK_Parameter( void )
							{
								if( mOwnData )
									delete [] mValue;
							} 

virtual	ulong 				GetSize( void ) const
							{
								return mSize;
							}

virtual	const uchar* 		GetDataPtr( void ) const
							{
								return mValue;
							}
							
virtual	long				GetParam( void ) const { return 0; }
							
							
	public://///////////////////////////////////////////////////////////////////////////////////
	
		void				SetData( 	
								uchar*	inValue, 
								ulong	inSize,
								bool	inDoCopy,
								bool	inOwnData ) 
							{
								mSize = inSize;

								if( (mCopy = inDoCopy) )
								{
									mValue = new uchar[inSize];
									memcpy( mValue, inValue, inSize );

									// we must always delete the data
									mOwnData = true;
								}
								else
								{
									mValue = inValue;
									mOwnData = inOwnData;
								}
							}


	private:///////////////////////////////////////////////////////////////////////////////////
	
		uchar*				mValue;
		ulong 				mSize;
		bool				mCopy;
		bool				mOwnData;
};


/**********************************************************************************************/
#endif // __VDK_Parameter_h

